@@ -1,5 +1,7 @@
---- Read and parse some of Nmap's data files: <code>nmap-protocols</code>,
--- <code>nmap-rpc</code>, and <code>nmap-services</code>.
+---
+-- Read and parse some of Nmap's data files: <code>nmap-protocols</code>,
+-- <code>nmap-rpc</code>, <code>nmap-services</code>, and 
+-- <code>nmap-mac-prefixes</code>.
 --
 -- The functions in this module return values appropriate for use with exception
 -- handling via <code>nmap.new_try</code>. On success, they return true and
@@ -8,9 +10,12 @@
 -- @author jah 08/2008
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
 
-module(... or "datafiles", package.seeall)
-
+local io = require "io"
+local nmap = require "nmap"
 local stdnse = require "stdnse"
+local string = require "string"
+local table = require "table"
+_ENV = stdnse.module("datafiles", stdnse.seeall)
 
 
 ---
@@ -23,10 +28,25 @@ local common_files = {
     ["nmap-protocols"] = { [function(ln) return tonumber( ln:match( "^%s*[^%s#]+%s+(%d+)" ) ) end] = "^%s*([^%s#]+)%s+%d+" },
     ["nmap-services"]  = { ["tcp"] = { [function(ln) return tonumber( ln:match( "^%s*[^%s#]+%s+(%d+)/tcp" ) ) end] = "^%s*([^%s#]+)%s+%d+/tcp" },
                            ["udp"] = { [function(ln) return tonumber( ln:match( "^%s*[^%s#]+%s+(%d+)/udp" ) ) end] = "^%s*([^%s#]+)%s+%d+/udp" }
-    }
+    },
+	["nmap-mac-prefixes"]	= { [ "^%s*(%w+)%s+[^#]+" ] = "^%s*%w+%s+([^#]+)" }
 
 }
 
+-- Helper for parse_* functions
+local parse_and_cache = function(filename)
+  nmap.registry.datafiles = nmap.registry.datafiles or {}
+  if not nmap.registry.datafiles[filename] then
+    local status
+    status, nmap.registry.datafiles[filename] = parse_file(filename)
+    if not status then
+      return false, string.format("Error parsing %s", filename)
+    end
+  end
+
+  return true, nmap.registry.datafiles[filename]
+end
+
 
 ---
 -- Read and parse <code>nmap-protocols</code>.
@@ -37,12 +57,7 @@ local common_files = {
 -- @return Table (if status is true) or error string (if status is false).
 -- @see parse_file
 parse_protocols = function()
-  local status, protocols_table = parse_file("nmap-protocols")
-  if not status then
-    return false, "Error parsing nmap-protocols"
-  end
-
-  return true, protocols_table
+  return parse_and_cache("nmap-protocols")
 end
 
 
@@ -54,12 +69,7 @@ end
 -- @return Table (if status is true) or error string (if status is false).
 -- @see parse_file
 parse_rpc = function()
-  local status, rpc_table = parse_file("nmap-rpc")
-  if not status then
-    return false, "Error parsing nmap-rpc"
-  end
-
-  return true, rpc_table
+  return parse_and_cache("nmap-rpc")
 end
 
 
@@ -81,15 +91,43 @@ parse_services = function(protocol)
     return false, "Bad protocol for nmap-services: use tcp or udp"
   end
 
-  local status, services_table = parse_file("nmap-services", protocol)
-  if not status then
-    return false, "Error parsing nmap-services"
+  local services_table
+  nmap.registry.datafiles = nmap.registry.datafiles or {}
+  nmap.registry.datafiles.services = nmap.registry.datafiles.services or {}
+  if protocol then
+    if not nmap.registry.datafiles.services[protocol] then
+      local status
+      status, nmap.registry.datafiles.services[protocol] = parse_file("nmap-services", protocol)
+      if not status then
+        return false, "Error parsing nmap-services"
+      end
+    end
+    services_table = nmap.registry.datafiles.services[protocol]
+  else
+    local status
+    status, nmap.registry.datafiles.services = parse_file("nmap-services")
+    if not status then
+      return false, "Error parsing nmap-services"
+    end
+    services_table = nmap.registry.datafiles.services
   end
 
   return true, services_table
 end
 
 
+---
+-- Read and parse <code>nmap-mac-prefixes</code>.
+--
+-- On success, return true and a table mapping 3 byte MAC prefixes to manufacturer names.
+-- @return Status (true or false).
+-- @return Table (if status is true) or error string (if status is false).
+-- @see parse_file
+parse_mac_prefixes = function()
+  return parse_and_cache("nmap-mac-prefixes")
+end
+
+
 ---
 -- Read and parse a generic data file. The other parse functions are
 -- defined in terms of this one.
@@ -147,7 +185,7 @@ function parse_file(filename, ...)
     if not status then return false, ret[#ret] end
   end
 
-  return true, unpack( ret )
+  return true, table.unpack( ret )
 
 end
 
@@ -225,10 +263,8 @@ function read_from_file( file )
     return false, ( "Error in datafiles.read_from_file: Cannot open %s for reading: %s" ):format( filepath, err )
   end
 
-  local line, ret = nil, {}
-  while true do
-    line = f:read()
-    if not line then break end
+  local ret = {}
+  for line in f:lines() do
     ret[#ret+1] = line
   end
 
@@ -287,3 +323,5 @@ get_assoc_array = function(lines, i_pattern, v_pattern)
   end
   return ret
 end
+
+return _ENV;
